@@ -16,7 +16,7 @@
 
 QT_USE_NAMESPACE
 
-Q_LOGGING_CATEGORY(lcAccessibilityTable, "qt.accessibility.table")
+Q_STATIC_LOGGING_CATEGORY(lcAccessibilityTable, "qt.accessibility.table")
 
 using namespace Qt::Literals::StringLiterals;
 
@@ -125,10 +125,6 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
                 if (iface->tableInterface()) {
                     [self updateTableModel];
                 } else if (const auto *cell = iface->tableCellInterface()) {
-                    // Called with role when populating a row with placeholder cells. The caller
-                    // inserts us into the correct array, so we don't have to look for it.
-                    if (role == NSAccessibilityCellRole)
-                        return self;
                     // If we create an element for a table cell, initialize it with row/column
                     // and insert it into the corresponding row's columns array.
                     m_rowIndex = cell->rowIndex();
@@ -553,8 +549,11 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
 }
 
 - (id) accessibilityWindow {
-    // We're in the same window as our parent.
-    return [self.accessibilityParent accessibilityWindow];
+    // Go up until we find a parent that is a window
+    NSAccessibilityElement *parent = self.accessibilityParent;
+    if (parent && parent.accessibilityRole == NSAccessibilityWindowRole)
+        return parent;
+    return [parent accessibilityWindow];
 }
 
 - (id) accessibilityTopLevelUIElementAttribute {
@@ -573,6 +572,26 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
     return nil;
 }
 
+- (id) accessibilityTitleUIElement {
+    QAccessibleInterface *iface = self.qtInterface;
+    if (!iface)
+        return nil;
+
+    const auto labelRelations = iface->relations(QAccessible::Label);
+    if (labelRelations.empty())
+        return nil;
+
+    QAccessibleInterface *label = labelRelations.first().first;
+    if (!label)
+        return nil;
+
+    QMacAccessibilityElement *accessibleElement = [QMacAccessibilityElement elementWithInterface:label];
+    if (!accessibleElement)
+        return nil;
+
+    return NSAccessibilityUnignoredAncestor(accessibleElement);
+}
+
 - (NSString*) accessibilityIdentifier {
     if (QAccessibleInterface *iface = self.qtInterface)
         return QAccessibleBridgeUtils::accessibleId(iface).toNSString();
@@ -605,11 +624,6 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
         return NSAccessibilityUnignoredAncestor([QMacAccessibilityElement elementWithId:axid]);
     }
 
-    // macOS expects that the hierarchy is:
-    // App -> Window -> Children
-    // We don't actually have the window reflected properly in QAccessibility.
-    // Check if the parent is the application and then instead return the native window.
-
     if (QAccessibleInterface *parent = iface->parent()) {
         if (parent->tableInterface()) {
             QMacAccessibilityElement *tableElement =
@@ -627,7 +641,12 @@ static void convertLineOffset(QAccessibleTextInterface *text, int *line, int *of
             QMacAccessibilityElement *rowElement = tableElement->rows[rowIndex];
             return NSAccessibilityUnignoredAncestor(rowElement);
         }
-        if (parent->role() != QAccessible::Application)
+        // macOS expects that the hierarchy is:
+        // App -> Window -> Children
+        // We don't actually have the window reflected properly in QAccessibility;
+        // the native framework does that for us. Check if the parent is the
+        // Application or a window, and if so return the native NSView instead.
+        if (parent->role() != QAccessible::Application && parent->role() != QAccessible::Window)
             return NSAccessibilityUnignoredAncestor([QMacAccessibilityElement elementWithInterface: parent]);
     }
 
